//package edu.unima.ky.parallel;

import java.io.*;
import java.net.*;

/**
 * ChannelFactory.
 * socket channel factory.
 *
 * Usage: ChannelFactory(port).
 * @author Akitoshi Yoshida
 * @author Heinz Kredel.
 */
public class ChannelFactory extends Thread {
  
/**
 * default port of socket. 
 */
  public final static int DEFAULT_PORT = 4711;

/**
 * port of socket. 
 */
  private int port;

/**
 * BoundedBuffer for sockets.
 */
  private BoundedBuffer buf = new BoundedBuffer(10);

/**
 * local server socket.
 */
  private ServerSocket srv = null;

/**
 * Constructs a ChannelFactory. 
 * @param p port.
 */
  public ChannelFactory(int p) {
    if (p<=0) { port = DEFAULT_PORT; } 
       else { port = p; }
    try {
        srv = new ServerSocket(port);
        this.start();
        System.out.println("server started on port "+port);
    } catch (IOException e) { System.out.println(e); }
  }

/**
 * Constructs a ChannelFactory.
 * on the DEFAULT_PORT.
 */
  public ChannelFactory() {
    this(DEFAULT_PORT); 
  }

/**
 * Get a new socket channel from a server socket.
 */
  public SocketChannel getChannel() 
         throws IOException {
    return (SocketChannel)buf.get();
  }

/**
 * Get a new socket channel to a given host.
 * @param h hostname.
 * @param p port.
 */
  public SocketChannel getChannel(String h, int p) 
         throws IOException {
    if (p<=0) { p = port; } 
    SocketChannel c = null;
    System.out.println("connecting to "+h);
    while (c == null) {
        try { c = new SocketChannel( new Socket(h, p) );
        } catch (IOException e) {
          System.out.println(e);
          // wait server ready
          System.out.println("Server on "+h+" not ready");
          try {
              Thread.currentThread().sleep((int)(5000));
          } catch (InterruptedException e1) { } 
        }
    }
    System.out.println("connected");
    return c;
  }

  public void run() {
    while (true) {
      try {
          System.out.println("waiting for connection");
          Socket s = srv.accept();
          System.out.println("connection accepted");
          SocketChannel c = new SocketChannel(s);
          buf.put( (Object) c );
      } catch (IOException e) {
	System.out.println("ChannelFactory terminating");
        return;
      }
    }
  }

/**
 * Terminate the Channel Factory.
 */
  public void terminate() {
    this.interrupt();
    try {
        srv.close();
        while (!buf.empty()) { 
            System.out.println("closing unused SocketChannel");
            ((SocketChannel)buf.get()).close();
        }
    } catch (IOException e) { }
  }

}

